import scala.sys.process._

organization := "aura"
name := "CM3D2_Batch_Installer"
version := "2.0"

licenses := Seq("CC0" -> url("https://directory.fsf.org/wiki/License:CC0"))

autoScalaLibrary := false
crossPaths := false

fork in run := true

mainClass in Compile := Some("aura.cm3d2.ui.Main")

libraryDependencies += "org.ini4j" % "ini4j" % "0.5.4"
libraryDependencies += "com.intellij" % "forms_rt" % "7.0.3"
libraryDependencies += "net.java.dev.jna" % "jna" % "4.5.1"

assemblyMergeStrategy in assembly := {
  case x if x.startsWith("com/sun/jna/") && x.contains("-") && !x.contains("win32-") =>
    MergeStrategy.discard
  case x if x.startsWith("com/intellij/uiDesigner") =>
    MergeStrategy.first
  case x =>
    val oldStrategy = (assemblyMergeStrategy in assembly).value
    oldStrategy(x)
}

val Version = "([0-9]+)\\.([0-9]+)(\\.([0-9]+))?".r
def system(cwd: File, p: String*) = {
  val ret = Process(p, cwd).!
  if(ret != 0) sys.error("process returned status code "+ret)
}
InputKey[Unit]("dist") := {
  val launch4j = crossTarget.value / "launch4j"
  if(!launch4j.exists()) launch4j.mkdirs()

  val jarFile = assembly.value
  val shortName = jarFile.getName.replaceAll("\\.jar$", "").replace("-assembly", "")
  val outFile = launch4j / s"$shortName.exe"
  val configPath = launch4j / "config.xml"

  val configPersister = net.sf.launch4j.config.ConfigPersister.getInstance()
  configPersister.load(baseDirectory.value / "project" / "launch4j.xml")
  val config = configPersister.getConfig
  
  config.setJar     (jarFile)
  config.setOutfile (outFile)
  config.setManifest(baseDirectory.value / "project" / "launch4j.manifest")

  config.setIcon(baseDirectory.value / "project" / "icon.ico")

  val Version(major, minor, _, patch) = version.value
  val win32Version = s"$major.$minor.${if(patch == null) "0" else patch}.0"
  config.getVersionInfo.setFileVersion      (win32Version)
  config.getVersionInfo.setTxtFileVersion   (version.value)
  config.getVersionInfo.setProductVersion   (win32Version)
  config.getVersionInfo.setTxtProductVersion(version.value)

  config.getVersionInfo.setOriginalFilename (s"$shortName.exe")
  config.getVersionInfo.setInternalName     (shortName)

  configPersister.save(configPath)

  system(baseDirectory.value, "/opt/launch4j/launch4j", configPath.getAbsolutePath)
  println("Final binary: "+outFile.getAbsolutePath)
}