package aura.cm3d2.core;

import aura.cm3d2.core.win32.Advapi32Util;
import aura.cm3d2.core.win32.WinReg;
import com.sun.jna.Platform;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;

public enum Game {
    CM3D2(
            "#449eff", // blue
            Utils.getString("Game.CM3D2.shortName"),
            "KISS", "カスタムメイド3D2", "CM3D2.exe",
            Utils.getString("Game.CM3D2.metadata.game"),
            Utils.getString("Game.CM3D2.metadata.category"),
            new String[] {
                    "カスタムメイド3D2",
            }
    ),
    CM3D2ChuB(
            "#00c2c5", // aqua
            Utils.getString("Game.CM3D2ChuB.shortName"),
            "KISS", "カスタムメイド3D2 with Chu-B Lip", "CM3D2OH.exe",
            Utils.getString("Game.CM3D2ChuB.metadata.game"),
            Utils.getString("Game.CM3D2ChuB.metadata.category"),
            new String[] {
                    "カスタムメイド3D2",
                    "カスタムメイド3D2 with Chu-B Lip",
                    "カスタムメイド3D2 with Chu-B Lipシリーズ",
            }
    ),
    COM3D2(
            "#ff6262", // red
            Utils.getString("Game.COM3D2.shortName"),
            "KISS", "カスタムオーダーメイド3D2", "COM3D2.exe",
            Utils.getString("Game.COM3D2.metadata.game"),
            Utils.getString("Game.COM3D2.metadata.category"),
            new String[] {
                    "カスタムオーダーメイド3D2",
            }
    ),
    COM3D2ChuB(
            "#ff7fde", // pink
            Utils.getString("Game.COM3D2ChuB.shortName"),
            "KISS", "カスタムオーダーメイド3D2 with Chu-B Lip", "COM3D2OH.exe",
            Utils.getString("Game.COM3D2ChuB.metadata.game"),
            Utils.getString("Game.COM3D2ChuB.metadata.category"),
            new String[] {
                    "カスタムオーダーメイド3D2",
                    "カスタムオーダーメイド3D2 with Chu-B Lip",
                    "カスタムオーダーメイド3D2 with Chu-B Lip3D2",
            }
    ),

    ;

    public final String displayColor;
    public final String shortName;

    public final String registryCompany;
    public final String registryProduct;
    public final String appExe;

    public final String mergeDisplayProduct;
    public final String mergeDisplayCategory;

    public final String[] displayProductAlias;

    Game(String displayColor, String shortName,
         String registryCompany, String registryProduct, String appExe,
         String mergeDisplayProduct, String mergeDisplayCategory,
         String[] displayProductAlias) {
        this.displayColor = displayColor;
        this.shortName = shortName;
        this.registryCompany = registryCompany;
        this.registryProduct = registryProduct;
        this.appExe = appExe;
        this.mergeDisplayProduct = mergeDisplayProduct;
        this.mergeDisplayCategory = mergeDisplayCategory;
        this.displayProductAlias = displayProductAlias;
    }

    public static Game checkGame(UpdateMetadata metadata) {
        for (Game game : values()) {
            if(metadata.isGame(game)) return game;
        }
        return null;
    }
    public static HashMap<Game, Path> getInstalledGames() {
        HashMap<Game, Path> installed = new HashMap<>();
        for (Game game: values()) {
            Path info = game.getInstallPath();
            if (info != null) {
                installed.put(game, info);
            }
        }
        return installed;
    }

    public Path getInstallPath() {
        if(!Platform.isWindows()) return null;

        try {
            String installPath = Advapi32Util.registryGetStringValue(
                    WinReg.HKEY_CURRENT_USER,
                    "Software\\"+registryCompany+"\\"+registryProduct,
                    "InstallPath"
            );
            return Paths.get(installPath);
        } catch(Exception e) {
            return null;
        }
    }
    public boolean isInstalled() {
        Path installPath = getInstallPath();
        return getInstallPath() != null && InstallState.exists(installPath);
    }
    public InstallState getInstallState() {
        Path installPath = getInstallPath();
        if (installPath == null) return null;
        return InstallState.open(installPath);
    }

    public UpdateMetadata getMergedMetadata() {
        return new UpdateMetadata(registryCompany, registryProduct, appExe, mergeDisplayProduct, mergeDisplayCategory);
    }
}
