package aura.cm3d2.core;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.function.Function;

class LockedFile {
    private final FileChannel channel;
    private final FileLock lock;

    public final Path path;

    private LockedFile(Path path, FileChannel channel, FileLock lock) {
        this.path = path;
        this.channel = channel;
        this.lock = lock;
    }

    public void release() {
        try {
            lock.release();
        } catch (IOException e) {
            System.err.println("Failed to release lock.");
            e.printStackTrace();
        }

        try {
            channel.close();
        } catch (IOException e) {
            System.err.println("Failed to close channel while releasing lock.");
            e.printStackTrace();
        }
    }

    public <T, U extends Comparable<? super U>> void writeList(T[] array,
                                                               Function<T, U> sortFn,
                                                               Function<T, String> fn) throws IOException {
        byte[] data = Utils.fromString(Utils.normalizeEndings(Utils.writeList(array, sortFn, fn)));
        channel.position(0);
        channel.truncate(0);
        channel.write(ByteBuffer.wrap(data));
    }

    public String[] getLines() throws IOException {
        channel.position(0);

        ByteArrayOutputStream data = new ByteArrayOutputStream();
        ByteBuffer dst = ByteBuffer.allocate(1024 * 16);

        int read;
        while ((read = channel.read(dst)) != -1) {
            data.write(dst.array(), 0, read);
            dst.position(0);
        }

        return Utils.splitLines(Utils.toString(data.toByteArray()));
    }

    public static LockedFile acquire(Path path) {
        try {
            FileChannel channel = FileChannel.open(path, StandardOpenOption.WRITE, StandardOpenOption.READ);
            FileLock lock = channel.tryLock();
            if(lock == null) return null;
            return new LockedFile(path, channel, lock);
        } catch (Exception e) {
            return null;
        }
    }
}
