package aura.cm3d2.core;

import aura.cm3d2.core.UpdatePackage.DisplayData;
import aura.cm3d2.core.UpdatePackage.DisplayTag;

import java.util.HashMap;

class Patches {
    private static class DisplayOverride {
        final DisplayData expected, override;

        private DisplayOverride(DisplayData expected, DisplayData override) {
            this.expected = expected;
            this.override = override;
        }
    }
    private static final HashMap<String, DisplayOverride> DISPLAY_OVERRIDE = new HashMap<>();
    private static void addDisplayOverride(String hash, DisplayData expected, DisplayData override) {
        DISPLAY_OVERRIDE.put(hash, new DisplayOverride(expected, override));
    }
    static {
        // In 2016, KISS seems to have had a streak of bonus packages with bad readmes.
        addDisplayOverride("lfRPokKzZ+Tdj2o2rmoaZdCjcShRCPU5KDepG954bWE=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "November 2016 TECH GIAN Bonus"));
        addDisplayOverride("2oVLtN4YFUoSKzpNFjo3dCaVu1YQTVRxrKFmdccKreQ=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "Denkigai 2016 Summer - Fan"));
        addDisplayOverride("o/N3wh18y+bgcu45warO8yHMXyQr5OwbV+WO9c2L5sk=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "Denkigai 2016 Summer - Incense Coaster + Skill"));
        addDisplayOverride("Q64fTl62sFDnEDuXA9Towp4dkq0CJ010Hk5UtPMFj94=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "Denkigai 2016 Summer - Leathercase"));
        addDisplayOverride("hREvhzTxVjon3wWRzaUCbR6eHmqgUePHncsKa7v/nlw=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "Denkigai 2016 Summer - Mastercard"));
        addDisplayOverride("OeaWlo0YCw+2jQ1pCJ135gGgjRBTBpyXCiseubDfaT4=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "Denkigai 2016 Summer - Mug"));
        addDisplayOverride("0oMTpUMyExz7MvDt/rSv2LWcM7MGu6b04/bFxq1MXbU=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "Denkigai 2016 Summer - School Swimsuit + Skill"));
        addDisplayOverride("oB+DtAeOLdQiRnVpvVvbeYKWXM6f4b/H33Y61fttII4=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "Denkigai 2016 Summer - Stand"));
        addDisplayOverride("JYtnuyAfpDZz2/D5vlF4Fwtt4mtyCNRhn9Ve/+XyUbs=",
                           new DisplayData(DisplayTag.UPDATER, "Ver1.38"),
                           new DisplayData(DisplayTag.PLUGIN, "Denkigai 2016 Summer - T-Shirt"));

        // anime-sharing.com repacks
        addDisplayOverride("m4cG3iizRXPXIOzjR9NTouaxky1/ALCo/7dQ8ApLsyk=",
                           new DisplayData(DisplayTag.UNKNOWN_REPACK, null),
                           new DisplayData(DisplayTag.REPACK, "オフィシャルファンブック"));
        addDisplayOverride("POSH+RbMRIhKbzlmQhAKF/nzq9vWW9Ncj/b5Toqeq8c=",
                           new DisplayData(DisplayTag.UNKNOWN, null),
                           new DisplayData(DisplayTag.REPACK, "Denkigai 2015 Winter - T-Shirt"));
        addDisplayOverride("5R4vm4CAsXm9VEB/icBbst2FkILURTodZHKSg9GroOI=",
                           new DisplayData(DisplayTag.UNKNOWN_REPACK, null),
                           new DisplayData(DisplayTag.REPACK, "カスタムメイド３Ｄ２ with Chu-B Lip プレイ追加"));
        addDisplayOverride("zKGMXAPsJzo4PgwfzZb2ivBQtVZZ7yESPRM4nr+XKrg=",
                           new DisplayData(DisplayTag.UNKNOWN_REPACK, null),
                           new DisplayData(DisplayTag.REPACK, "ビジュアルパック"));

        // I'm not sure if these are repacks, or just KISS using an unusual readme format.
        addDisplayOverride("cu5iv+kWYW/Skh/LqPNMXJxIBkkgcHhfaKQnjvN/qe8=",
                           new DisplayData(DisplayTag.UNKNOWN, null),
                           new DisplayData(DisplayTag.PLUGIN, "夜伽クラス　Vol.3　セックススレイブメイド"));
        addDisplayOverride("Qlz4Daw488Wv0GFC6WdctQqd8Nf3jnnG4CmQXofKrYU=",
                           new DisplayData(DisplayTag.UNKNOWN, null),
                           new DisplayData(DisplayTag.PLUGIN, "DLCセット Vol.7"));
    }

    public static DisplayData patchDisplayData(UpdatePackage pkg) {
        DisplayOverride override = DISPLAY_OVERRIDE.get(pkg.fileEntryHash());
        if (override != null &&
                (override.expected == null ||
                 override.expected.tag == pkg.display.tag ||
                 override.expected.name == null ||
                 override.expected.equals(pkg.display))) {
            System.out.println("   - Unpatched name: "+pkg.display);
            return new DisplayData(
                    override.override.tag  == null ? pkg.display.tag  : override.override.tag,
                    override.override.name == null ? pkg.display.name : override.override.name
            );
        } else {
            return pkg.display;
        }
    }
}
