package aura.cm3d2.core;

import org.ini4j.Ini;
import org.ini4j.Profile;
import org.ini4j.Wini;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.file.Files;
import java.nio.file.Path;

public class UpdateMetadata {
    public final String registryCompany;
    public final String registryProduct;
    public final String appExe;

    public final String displayProduct;
    public final String displayCategory;

    public UpdateMetadata(String registryCompany, String registryProduct, String appExe,
                          String displayProduct, String displayCategory) {
        this.registryCompany = registryCompany;
        this.registryProduct = registryProduct;
        this.appExe = appExe;
        this.displayProduct = displayProduct;
        this.displayCategory = displayCategory;
    }

    public static UpdateMetadata read(Path path) throws IOException {
        Ini ini = new Wini();
        ini.load(new InputStreamReader(Files.newInputStream(path), Utils.SJIS));

        Profile.Section section = ini.get("UPDATER");

        String company = section.get("Company");
        String product = section.get("Registry1");
        String appExe  = section.get("AppExe");

        String displayProduct  = section.get("AppName");
        String displayCategory = section.get("Category");

        return new UpdateMetadata(company, product, appExe, displayProduct, displayCategory);
    }

    public void write(Path path) throws IOException {
        Ini ini = new Wini();

        ini.put("UPDATER", "Company"  , registryCompany);
        ini.put("UPDATER", "Registry1", registryProduct);
        ini.put("UPDATER", "AppExe"   , appExe);

        ini.put("UPDATER", "AppName"  , displayProduct);
        ini.put("UPDATER", "Category" , displayCategory);

        OutputStreamWriter out = new OutputStreamWriter(Files.newOutputStream(path), Utils.SJIS);
        ini.store(out);
        out.close();
    }

    public boolean isGame(Game game) {
        return game.registryCompany.equals(registryCompany) &&
                game.registryProduct.equals(registryProduct) &&
                game.appExe.equalsIgnoreCase(appExe);
    }
}
