package aura.cm3d2.core;

import org.boris.pecoff4j.PE;
import org.boris.pecoff4j.ResourceEntry;
import org.boris.pecoff4j.constant.ResourceType;
import org.boris.pecoff4j.io.PEParser;
import org.boris.pecoff4j.io.ResourceParser;
import org.boris.pecoff4j.resources.*;
import org.boris.pecoff4j.util.ResourceHelper;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

public class UpdaterBinary {
    public final Path source;
    public final String productName;
    public final long productVersion;

    public static UpdaterBinary read(Path source) throws IOException {
        PE pe = PEParser.parse(Files.newInputStream(source));
        ResourceEntry[] resourceEntries =
                ResourceHelper.findResources(pe.getImageData().getResourceTable(), ResourceType.VERSION_INFO);
        for(ResourceEntry resource : resourceEntries) {
			VersionInfo version = ResourceParser.readVersionInfo(resource.getData());
            FixedFileInfo info = version.getFixedFileInfo();

            String productName = "<unknown>";

            StringFileInfo strInfo = version.getStringFileInfo();
            int tableCount = strInfo.getCount();
            for(int i=0; i<tableCount; i++) {
                StringTable table = strInfo.getTable(i);
                int entryCount = table.getCount();
                for(int j=0; j<entryCount; j++) {
                    StringPair entry = table.getString(j);
                    if(entry.getKey().equalsIgnoreCase("ProductName"))
                        productName = entry.getValue();
                }
            }

            return new UpdaterBinary(source, productName,
                                     (((long) info.getFileVersionMS()) << 32) | info.getFileVersionLS());
        }
        throw new IOException("Could not parse PE file.");
    }

    private UpdaterBinary(Path source, String productName, long productVersion) {
        this.source = source;
        this.productName = productName;
        this.productVersion = productVersion;
    }

    @Override
    public String toString() {
        return "UpdaterBinary{" +
                "source=" + source +
                ", productName='" + productName + '\'' +
                ", productVersion=" + Long.toHexString(productVersion) +
                '}';
    }
}
