/* Copyright (c) 2010 Daniel Doubrovkine, All Rights Reserved
 *
 * The contents of this file is dual-licensed under 2 
 * alternative Open Source/Free licenses: LGPL 2.1 or later and 
 * Apache License 2.0. (starting with JNA version 4.0.0).
 * 
 * You can freely decide which license you want to apply to 
 * the project.
 * 
 * You may obtain a copy of the LGPL License at:
 * 
 * http://www.gnu.org/licenses/licenses.html
 * 
 * A copy is also included in the downloadable source code package
 * containing JNA, in file "LGPL2.1".
 * 
 * You may obtain a copy of the Apache License at:
 * 
 * http://www.apache.org/licenses/
 * 
 * A copy is also included in the downloadable source code package
 * containing JNA, in file "AL2.0".
 */
package aura.cm3d2.core.win32;

import com.sun.jna.ptr.IntByReference;
import com.sun.jna.win32.StdCallLibrary;

/**
 * Advapi32.dll Interface.
 *
 * @author dblock[at]dblock.org
 */
public interface Advapi32 extends StdCallLibrary {
    /**
     * The RegOpenKeyEx function opens the specified registry key. Note that key
     * names are not case sensitive.
     *
     * @param hKey
     *            Handle to an open key.
     * @param lpSubKey
     *            Pointer to a null-terminated string containing the name of the
     *            subkey to open.
     * @param ulOptions
     *            Reserved; must be zero.
     * @param samDesired
     *            Access mask that specifies the desired access rights to the
     *            key. The function fails if the security descriptor of the key
     *            does not permit the requested access for the calling process.
     * @param phkResult
     *            Pointer to a variable that receives a handle to the opened
     *            key. If the key is not one of the predefined registry keys,
     *            call the RegCloseKey function after you have finished using
     *            the handle.
     * @return If the function succeeds, the return value is ERROR_SUCCESS. If
     *         the function fails, the return value is a nonzero error code
     *         defined in Winerror.h.
     */
    int RegOpenKeyEx(WinReg.HKEY hKey, String lpSubKey, int ulOptions,
                     int samDesired, WinReg.HKEYByReference phkResult);

    /**
     * The RegQueryValueEx function retrieves the type and data for a specified
     * value name associated with an open registry key.
     *
     * @param hKey
     *            Handle to an open key. The key must have been opened with the
     *            KEY_QUERY_VALUE access right.
     * @param lpValueName
     *            Pointer to a null-terminated string containing the name of the
     *            value to query. If lpValueName is NULL or an empty string, "",
     *            the function retrieves the type and data for the key's unnamed
     *            or default value, if any.
     * @param lpReserved
     *            Reserved; must be NULL.
     * @param lpType
     *            Pointer to a variable that receives a code indicating the type
     *            of data stored in the specified value.
     * @param lpData
     *            Pointer to a buffer that receives the value's data. This
     *            parameter can be NULL if the data is not required. If the data
     *            is a string, the function checks for a terminating null
     *            character. If one is not found, the string is stored with a
     *            null terminator if the buffer is large enough to accommodate
     *            the extra character. Otherwise, the string is stored as is.
     * @param lpcbData
     *            Pointer to a variable that specifies the size of the buffer
     *            pointed to by the lpData parameter, in bytes. When the
     *            function returns, this variable contains the size of the data
     *            copied to lpData. The lpcbData parameter can be NULL only if
     *            lpData is NULL. If the data has the REG_SZ, REG_MULTI_SZ or
     *            REG_EXPAND_SZ type, this size includes any terminating null
     *            character or characters. If the buffer specified by lpData
     *            parameter is not large enough to hold the data, the function
     *            returns ERROR_MORE_DATA and stores the required buffer size in
     *            the variable pointed to by lpcbData. In this case, the
     *            contents of the lpData buffer are undefined. If lpData is
     *            NULL, and lpcbData is non-NULL, the function returns
     *            ERROR_SUCCESS and stores the size of the data, in bytes, in
     *            the variable pointed to by lpcbData. This enables an
     *            application to determine the best way to allocate a buffer for
     *            the value's data.
     * @return If the function succeeds, the return value is ERROR_SUCCESS. If
     *         the function fails, the return value is a nonzero error code
     *         defined in Winerror.h.
     */
    int RegQueryValueEx(WinReg.HKEY hKey, String lpValueName, int lpReserved,
                        IntByReference lpType, char[] lpData, IntByReference lpcbData);

    /**
     * The RegCloseKey function releases a handle to the specified registry key.
     *
     * @param hKey
     *            Handle to the open key to be closed. The handle must have been
     *            opened by the RegCreateKeyEx, RegOpenKeyEx, or
     *            RegConnectRegistry function.
     * @return If the function succeeds, the return value is ERROR_SUCCESS. If
     *         the function fails, the return value is a nonzero error code
     *         defined in Winerror.h.
     */
    int RegCloseKey(WinReg.HKEY hKey);
}
