package aura.cm3d2.core.win32;

import com.sun.jna.Pointer;
import com.sun.jna.Structure;
import com.sun.jna.Union;

import java.util.Arrays;
import java.util.List;

/**
     * Contains information about the current computer system. This includes the architecture and
     * type of the processor, the number of processors in the system, the page size, and other such
     * information.
     */
public class SYSTEM_INFO extends Structure {
    public static final int PROCESSOR_ARCHITECTURE_AMD64 = 9;

    /** Unnamed inner structure. */
    public static class PI extends Structure {

        public static class ByReference extends PI implements Structure.ByReference {

        }

        /**
         * System's processor architecture.
         * This value can be one of the following values:
         *
         *  PROCESSOR_ARCHITECTURE_UNKNOWN
         *  PROCESSOR_ARCHITECTURE_INTEL
         *  PROCESSOR_ARCHITECTURE_IA64
         *  PROCESSOR_ARCHITECTURE_AMD64
         */
        public WinDef.WORD wProcessorArchitecture;
        /**
         * Reserved for future use.
         */
        public WinDef.WORD wReserved;

        @Override
        protected List<String> getFieldOrder() {
            return Arrays.asList("wProcessorArchitecture", "wReserved");
        }
    }

    /** Unnamed inner union. */
    public static class UNION extends Union {

        public static class ByReference extends UNION implements Structure.ByReference {

        }

        /**
         * An obsolete member that is retained for compatibility with Windows NT 3.5 and earlier.
         * New applications should use the wProcessorArchitecture branch of the union.
         * Windows Me/98/95: The system always sets this member to zero, the value defined
         * for PROCESSOR_ARCHITECTURE_INTEL.
         */
        public WinDef.DWORD dwOemID;
        /**
         * Processor architecture (unnamed struct).
         */
        public PI pi;
    }

    /**
     * Processor architecture (unnamed union).
     */
    public UNION processorArchitecture;
    /**
     * Page size and the granularity of page protection and commitment.
     */
    public WinDef.DWORD dwPageSize;
    /**
     * Pointer to the lowest memory address accessible to applications and dynamic-link libraries (DLLs).
     */
    public Pointer lpMinimumApplicationAddress;
    /**
     * Pointer to the highest memory address accessible to applications and DLLs.
     */
    public Pointer lpMaximumApplicationAddress;
    /**
     * Mask representing the set of processors configured into the system. Bit 0 is processor 0; bit 31 is processor 31.
     */
    public WinDef.DWORD_PTR dwActiveProcessorMask;
    /**
     * Number of processors in the system.
     */
    public WinDef.DWORD dwNumberOfProcessors;
    /**
     * An obsolete member that is retained for compatibility with Windows NT 3.5 and Windows Me/98/95.
     * Use the wProcessorArchitecture, wProcessorLevel, and wProcessorRevision members to determine
     * the type of processor.
     *  PROCESSOR_INTEL_386
     *  PROCESSOR_INTEL_486
     *  PROCESSOR_INTEL_PENTIUM
     */
    public WinDef.DWORD dwProcessorType;
    /**
     * Granularity for the starting address at which virtual memory can be allocated.
     */
    public WinDef.DWORD dwAllocationGranularity;
    /**
     * System's architecture-dependent processor level. It should be used only for display purposes.
     * To determine the feature set of a processor, use the IsProcessorFeaturePresent function.
     * If wProcessorArchitecture is PROCESSOR_ARCHITECTURE_INTEL, wProcessorLevel is defined by the CPU vendor.
     * If wProcessorArchitecture is PROCESSOR_ARCHITECTURE_IA64, wProcessorLevel is set to 1.
     */
    public WinDef.WORD wProcessorLevel;
    /**
     * Architecture-dependent processor revision.
     */
    public WinDef.WORD wProcessorRevision;

    @Override
    protected List<String> getFieldOrder() {
        return Arrays.asList("processorArchitecture", "dwPageSize", "lpMinimumApplicationAddress", "lpMaximumApplicationAddress", "dwActiveProcessorMask", "dwNumberOfProcessors", "dwProcessorType", "dwAllocationGranularity", "wProcessorLevel", "wProcessorRevision");
    }
}
