/* Copyright (c) 2010 Daniel Doubrovkine, All Rights Reserved
 *
 * The contents of this file is dual-licensed under 2 
 * alternative Open Source/Free licenses: LGPL 2.1 or later and 
 * Apache License 2.0. (starting with JNA version 4.0.0).
 * 
 * You can freely decide which license you want to apply to 
 * the project.
 * 
 * You may obtain a copy of the LGPL License at:
 * 
 * http://www.gnu.org/licenses/licenses.html
 * 
 * A copy is also included in the downloadable source code package
 * containing JNA, in file "LGPL2.1".
 * 
 * You may obtain a copy of the Apache License at:
 * 
 * http://www.apache.org/licenses/
 * 
 * A copy is also included in the downloadable source code package
 * containing JNA, in file "AL2.0".
 */
package aura.cm3d2.core.win32;

import com.sun.jna.IntegerType;
import com.sun.jna.Native;
import com.sun.jna.Pointer;

/**
 * Ported from Windef.h (various macros and types). Microsoft Windows SDK 6.0A.
 *
 * @author dblock[at]dblock.org
 */
@SuppressWarnings("serial")
public interface WinDef {
    /**
     * 16-bit unsigned integer.
     */
    class WORD extends IntegerType implements Comparable<WORD> {

        /** The Constant SIZE. */
        public static final int SIZE = 2;

        /**
         * Instantiates a new word.
         */
        public WORD() {
            this(0);
        }

        /**
         * Instantiates a new word.
         *
         * @param value
         *            the value
         */
        public WORD(long value) {
            super(SIZE, value, true);
        }

        @Override
        public int compareTo(WORD other) {
            return compare(this, other);
        }
    }

    /**
     * 32-bit unsigned integer.
     */
    class DWORD extends IntegerType implements Comparable<DWORD> {

        /** The Constant SIZE. */
        public static final int SIZE = 4;

        /**
         * Instantiates a new dword.
         */
        public DWORD() {
            this(0);
        }

        /**
         * Instantiates a new dword.
         *
         * @param value
         *            the value
         */
        public DWORD(long value) {
            super(SIZE, value, true);
        }

        /**
         * Low WORD.
         *
         * @return Low WORD.
         */
        public WORD getLow() {
            return new WORD(longValue() & 0xFFFF);
        }

        /**
         * High WORD.
         *
         * @return High WORD.
         */
        public WORD getHigh() {
            return new WORD((longValue() >> 16) & 0xFFFF);
        }

        @Override
        public int compareTo(DWORD other) {
            return compare(this, other);
        }
    }

    /**
     * The Class LONG.
     */
    class LONG extends IntegerType implements Comparable<LONG> {

        /** The Constant SIZE. */
        public static final int SIZE = Native.LONG_SIZE;

        /**
         * Instantiates a new long.
         */
        public LONG() {
            this(0);
        }

        /**
         * Instantiates a new long.
         *
         * @param value the value
         */
        public LONG(long value) {
            super(SIZE, value);
        }

        @Override
        public int compareTo(LONG other) {
            return compare(this, other);
        }
    }

    /**
     * The Class LONGLONG.
     */
    class LONGLONG extends IntegerType implements Comparable<LONGLONG> {

        /** The Constant SIZE. */
        public static final int SIZE = Native.LONG_SIZE *2;

        /**
         * Instantiates a new LONGLONG.
         */
        public LONGLONG() {
            this(0);
        }

        /**
         * Instantiates a new LONGLONG.
         *
         * @param value the value
         */
        public LONGLONG(long value) {
            super(8, value, false);
        }

        @Override
        public int compareTo(LONGLONG other) {
            return compare(this, other);
        }
    }

    class DWORD_PTR extends IntegerType {
        public DWORD_PTR() {
            this(0);
        }

        public DWORD_PTR(long value) {
            super(Pointer.SIZE, value);
        }
    }
}