package aura.cm3d2.ui;

import aura.cm3d2.core.TaskList;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.io.IOException;
import java.util.ArrayList;
import java.util.ResourceBundle;

public class Main {
    static final ResourceBundle TEXT = ResourceBundle.getBundle("aura/cm3d2/ui/text");
    static final String TITLE = TEXT.getString("CM3D2BatchInstaller.title");

    static final ArrayList<Image> ICONS = new ArrayList<>();
    static {
        try {
            ICONS.add(ImageIO.read(Main.class.getResource("icon-256.png")));
            ICONS.add(ImageIO.read(Main.class.getResource("icon-128.png")));
            ICONS.add(ImageIO.read(Main.class.getResource("icon-64.png")));
            ICONS.add(ImageIO.read(Main.class.getResource("icon-48.png")));
            ICONS.add(ImageIO.read(Main.class.getResource("icon-32.png")));
            ICONS.add(ImageIO.read(Main.class.getResource("icon-16.png")));
            ICONS.add(ImageIO.read(Main.class.getResource("icon-8.png")));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static void showUntranslatedMessage(Window parent, int messageType, String msg) {
        JOptionPane.showMessageDialog(parent, msg, Main.TITLE, messageType);
    }
    static void showMessage(Window parent, int messageType, String i18n) {
        showUntranslatedMessage(parent, messageType, TEXT.getString(i18n));
    }
    static void showFormatMessage(Window parent, int messageType, String i18n, Object... format) {
        showUntranslatedMessage(parent, messageType, String.format(TEXT.getString(i18n), format));
    }

    static int showUntranslatedConfirmDialog(Window parent, int optionType, int messageType, String msg) {
        return JOptionPane.showConfirmDialog(parent, msg, Main.TITLE, optionType, messageType);
    }
    static int showConfirmDialog(Window parent, int optionType, int messageType, String i18n) {
        return showUntranslatedConfirmDialog(parent, optionType, messageType, TEXT.getString(i18n));
    }
    static int showConfirmDialog(Window parent, int optionType, int messageType, String i18n, Object... format) {
        return showUntranslatedConfirmDialog(parent, optionType, messageType,
                String.format(TEXT.getString(i18n), format));
    }

    public static void main(String[] args) {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException | InstantiationException |
                IllegalAccessException | UnsupportedLookAndFeelException e) {
            e.printStackTrace();
        }

        TaskList.enableDebugPrint();

        MainDialog dialog = new MainDialog();
        dialog.pack();
        dialog.setLocationRelativeTo(null);
        dialog.setVisible(true);
    }
}
