package aura.cm3d2.ui;

import aura.cm3d2.core.*;
import aura.cm3d2.core.Game;
import com.intellij.uiDesigner.core.GridConstraints;
import com.intellij.uiDesigner.core.GridLayoutManager;
import com.intellij.uiDesigner.core.Spacer;
import com.sun.jna.Platform;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.nio.file.Path;
import java.util.ResourceBundle;

class MainDialog extends JFrame {
    private JPanel contentPane;
    private JList<String> listUpdatePackages;
    private JButton addButton;
    private JButton removeButton;
    private JButton mergeButton;
    private JButton settingsButton;
    private JButton installButton;

    private SettingsDialog settings = new SettingsDialog(this);
    private ArchiveList list = new ArchiveList();

    MainDialog() {
        setContentPane(contentPane);
        setTitle(Main.TITLE);
        setIconImages(Main.ICONS);

        setMinimumSize(new Dimension(600, 400));
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

        updateArchiveList();

        addButton.addActionListener(event -> withUpdaterGuard(this::addAction));
        removeButton.addActionListener(event -> removeAction());
        listUpdatePackages.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_DELETE) removeAction();
            }
        });
        settingsButton.addActionListener(event -> settingsAction());
        mergeButton.addActionListener(event -> withUpdaterGuard(this::mergeAction));

        if (!Platform.isWindows())
            // TODO: Support WINE installations
            installButton.setVisible(false);
        else {
            installButton.setEnabled(true);
            installButton.addActionListener(event -> withUpdaterGuard(this::installAction));
        }

        listUpdatePackages.addMouseMotionListener(new MouseMotionAdapter() {
            @Override
            public void mouseMoved(MouseEvent e) {
                ListModel m = listUpdatePackages.getModel();
                int index = listUpdatePackages.locationToIndex(e.getPoint());
                if(index > -1) listUpdatePackages.setToolTipText(currentPathIndicies[index].path.toString());
                else listUpdatePackages.setToolTipText(null);
            }
        });
    }

    private ArchiveList.PathEntry[] currentPathIndicies;

    private void updateArchiveList() {
        DefaultListModel<String> model = new DefaultListModel<>();
        currentPathIndicies = list.getPaths();
        for (ArchiveList.PathEntry entry : currentPathIndicies) {
            model.addElement(String.format("<html><font color=%s><b>[%s]</b></font> %s</html>",
                    entry.pkg.game.displayColor, entry.pkg.game.shortName, entry.pkg.display.toHtmlString()));
        }
        listUpdatePackages.setModel(model);
    }

    private JFileChooser setupChooser() {
        JFileChooser chooser = new JFileChooser() {
            {
                setIconImages(Main.ICONS);
            }
        };
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setAcceptAllFileFilterUsed(false);
        return chooser;
    }

    {
// GUI initializer generated by IntelliJ IDEA GUI Designer
// >>> IMPORTANT!! <<<
// DO NOT EDIT OR ADD ANY CODE HERE!
        $$$setupUI$$$();
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection ALL
     */
    private void $$$setupUI$$$() {
        contentPane = new JPanel();
        contentPane.setLayout(new GridLayoutManager(1, 1, new Insets(10, 10, 10, 10), -1, -1));
        final JPanel panel1 = new JPanel();
        panel1.setLayout(new GridLayoutManager(1, 2, new Insets(0, 0, 0, 0), -1, -1));
        contentPane.add(panel1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        final JScrollPane scrollPane1 = new JScrollPane();
        panel1.add(scrollPane1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
        listUpdatePackages = new JList();
        scrollPane1.setViewportView(listUpdatePackages);
        final JPanel panel2 = new JPanel();
        panel2.setLayout(new GridLayoutManager(6, 1, new Insets(0, 0, 0, 0), -1, -1));
        panel1.add(panel2, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        addButton = new JButton();
        this.$$$loadButtonText$$$(addButton, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.add"));
        addButton.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.add.tooltip"));
        addButton.putClientProperty("html.disable", Boolean.TRUE);
        panel2.add(addButton, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        removeButton = new JButton();
        this.$$$loadButtonText$$$(removeButton, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.remove"));
        removeButton.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.remove.tooltip"));
        removeButton.putClientProperty("html.disable", Boolean.TRUE);
        panel2.add(removeButton, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        mergeButton = new JButton();
        this.$$$loadButtonText$$$(mergeButton, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.merge"));
        mergeButton.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.merge.tooltip"));
        mergeButton.putClientProperty("html.disable", Boolean.TRUE);
        panel2.add(mergeButton, new GridConstraints(5, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final Spacer spacer1 = new Spacer();
        panel2.add(spacer1, new GridConstraints(3, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
        settingsButton = new JButton();
        this.$$$loadButtonText$$$(settingsButton, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.settings"));
        settingsButton.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.settings.tooltip"));
        settingsButton.putClientProperty("html.disable", Boolean.TRUE);
        panel2.add(settingsButton, new GridConstraints(2, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        installButton = new JButton();
        installButton.setEnabled(false);
        this.$$$loadButtonText$$$(installButton, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.install"));
        installButton.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("MainDialog.install.tooltip"));
        installButton.putClientProperty("html.disable", Boolean.TRUE);
        panel2.add(installButton, new GridConstraints(4, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    }

    /**
     * @noinspection ALL
     */
    private void $$$loadButtonText$$$(AbstractButton component, String text) {
        StringBuffer result = new StringBuffer();
        boolean haveMnemonic = false;
        char mnemonic = '\0';
        int mnemonicIndex = -1;
        for (int i = 0; i < text.length(); i++) {
            if (text.charAt(i) == '&') {
                i++;
                if (i == text.length()) break;
                if (!haveMnemonic && text.charAt(i) != '&') {
                    haveMnemonic = true;
                    mnemonic = text.charAt(i);
                    mnemonicIndex = result.length();
                }
            }
            result.append(text.charAt(i));
        }
        component.setText(result.toString());
        if (haveMnemonic) {
            component.setMnemonic(mnemonic);
            component.setDisplayedMnemonicIndex(mnemonicIndex);
        }
    }

    /**
     * @noinspection ALL
     */
    public JComponent $$$getRootComponent$$$() {
        return contentPane;
    }

    interface ProgressAction {
        void run(IProgressListener progress) throws Exception;
    }

    private void withUpdaterGuard(Runnable runnable) {
        try {
            runnable.run();
        } catch (UpdaterException e) {
            Main.showUntranslatedMessage(this, JOptionPane.ERROR_MESSAGE, e.getMessage());
        }
    }

    private boolean withProgressDialog(ProgressAction fn) {
        ProgressDialog dialog = new ProgressDialog();
        Thread runThread = new Thread(() -> {
            try {
                fn.run(dialog);
            } catch (CanceledError e) {
                dialog.dispose();
            } catch (UpdaterException e) {
                Main.showUntranslatedMessage(this, JOptionPane.ERROR_MESSAGE, e.getMessage());
                dialog.errorOccurred();
            } catch (Exception e) {
                Main.showFormatMessage(this, JOptionPane.ERROR_MESSAGE, "MainDialog.unexpectedError",
                        e.getClass().toString(), e.getMessage());
                e.printStackTrace();
                dialog.errorOccurred();
            }
        });
        runThread.start();
        dialog.pack();
        dialog.setLocationRelativeTo(this);
        dialog.setVisible(true);
        return !dialog.hasErrorOccurred() && !dialog.canceled();
    }

    private void runTaskList(TaskList tasks, String onSuccess, String onCancel) {
        boolean[] canceled = new boolean[1];
        if (withProgressDialog(progress -> {
            try {
                tasks.execute(progress, settings.getSettings());
            } catch (CanceledError e) {
                canceled[0] = true;
                throw e;
            }
        })) Main.showMessage(this, JOptionPane.PLAIN_MESSAGE, onSuccess);
        else if (canceled[0]) Main.showMessage(this, JOptionPane.WARNING_MESSAGE, onCancel);
    }

    private void addAction() {
        JFileChooser chooser = setupChooser();
        if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            int[] countPtr = new int[1];
            if (withProgressDialog(progress ->
                    countPtr[0] = list.findPackagesInDirectory(progress, chooser.getSelectedFile().toPath()))) {
                int count = countPtr[0];
                if (count == 0) Main.showMessage(this, JOptionPane.ERROR_MESSAGE, "MainDialog.noUpdates");
                else Main.showFormatMessage(this, JOptionPane.PLAIN_MESSAGE, "MainDialog.foundUpdates", count);
            }
            updateArchiveList();
        }
    }

    private void removeAction() {
        for (int i : listUpdatePackages.getSelectedIndices()) list.removePath(currentPathIndicies[i].path);
        updateArchiveList();
    }

    private void settingsAction() {
        settings.pack();
        settings.setLocationRelativeTo(this);
        settings.setVisible(true);
    }

    private boolean checkPackageCount() {
        if (list.getPaths().length == 0) {
            Main.showMessage(this, JOptionPane.PLAIN_MESSAGE, "MainDialog.noPackagesSelected");
            return false;
        }
        return true;
    }

    private boolean checkGamesSupported() {
        StringBuilder missing = new StringBuilder();
        long supportedCount = 0, unsupportedCount = 0;
        for (Game game : Game.values()) {
            long count = list.getPackageCount(game);
            if (count > 0) {
                if (game.isInstalled()) {
                    supportedCount += count;
                } else {
                    missing.append("\n• ").append(game.mergeDisplayProduct);
                    unsupportedCount += count;
                }
            }
        }

        if (unsupportedCount == 0) return true;
        else if (supportedCount == 0) { // unsupportedCount > 0, supportedCount == 0
            Main.showFormatMessage(this, JOptionPane.PLAIN_MESSAGE, "MainDialog.noSupportedPackagesSelected",
                    unsupportedCount, missing);
            return false;
        } else {
            return Main.showConfirmDialog(this, JOptionPane.YES_NO_OPTION, JOptionPane.PLAIN_MESSAGE,
                    "MainDialog.someSupported",
                    unsupportedCount, missing, supportedCount) == JOptionPane.YES_OPTION;
        }
    }

    private void checkExtendedVerification(TaskList tasks) {
        if (settings.getEnableExtendedValidation()) {
            for (UpdatePackage pkg : list.getPackages()) {
                pkg.verifyPackage(tasks);
            }
        }
    }

    private void installAction() {
        if (!checkPackageCount()) return;
        if (!checkGamesSupported()) return;
        TaskList tasks = UpdatePackage.installPackages(list.getPackages());
        checkExtendedVerification(tasks);
        runTaskList(tasks, "MainDialog.installSuccessful", "MainDialog.updateCanceled");
    }

    private void mergeAction() {
        if (!checkPackageCount()) return;

        JFileChooser chooser = setupChooser();
        if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
            Path outPath = chooser.getSelectedFile().toPath();
            TaskList tasks = UpdatePackage.mergePackages(outPath, list.getPackages());
            checkExtendedVerification(tasks);
            runTaskList(tasks, "MainDialog.mergeSuccessful", "MainDialog.mergeCanceled");
        }
    }
}
