package aura.cm3d2.ui;

import aura.cm3d2.core.Settings;
import com.intellij.uiDesigner.core.GridConstraints;
import com.intellij.uiDesigner.core.GridLayoutManager;
import com.intellij.uiDesigner.core.Spacer;
import com.sun.jna.Platform;

import javax.swing.*;
import java.awt.*;
import java.util.ResourceBundle;

class SettingsDialog extends JDialog {
    private JPanel contentPane;
    private JButton buttonOK;
    private JCheckBox extendedValidationCheckbox;
    private JCheckBox disableDiskSpaceCheckingCheckbox;
    private JCheckBox skipChecksumValidationCheckBox;
    private JCheckBox dontMerge32BitAndCheckBox;
    private JCheckBox registerInstalledUpdateFilesCheckBox;

    SettingsDialog(Window parent) {
        super(parent);

        setContentPane(contentPane);
        setModal(true);
        getRootPane().setDefaultButton(buttonOK);

        setIconImages(Main.ICONS);

        buttonOK.addActionListener(event -> setVisible(false));

        setMinimumSize(new Dimension(250, 400));
        setDefaultCloseOperation(HIDE_ON_CLOSE);

        // Disk space checking is broken on *nix, so disable disk space checking by default.
        if (!Platform.isWindows()) disableDiskSpaceCheckingCheckbox.setSelected(true);
    }

    boolean getEnableExtendedValidation() {
        return extendedValidationCheckbox.isSelected();
    }

    Settings getSettings() {
        return new Settings(
                disableDiskSpaceCheckingCheckbox.isSelected(),
                skipChecksumValidationCheckBox.isSelected(),
                dontMerge32BitAndCheckBox.isSelected(),
                registerInstalledUpdateFilesCheckBox.isSelected());
    }

    {
// GUI initializer generated by IntelliJ IDEA GUI Designer
// >>> IMPORTANT!! <<<
// DO NOT EDIT OR ADD ANY CODE HERE!
        $$$setupUI$$$();
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection ALL
     */
    private void $$$setupUI$$$() {
        contentPane = new JPanel();
        contentPane.setLayout(new GridLayoutManager(2, 1, new Insets(10, 10, 10, 10), -1, -1));
        final JPanel panel1 = new JPanel();
        panel1.setLayout(new GridLayoutManager(1, 2, new Insets(0, 0, 0, 0), -1, -1));
        contentPane.add(panel1, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, 1, null, null, null, 0, false));
        final Spacer spacer1 = new Spacer();
        panel1.add(spacer1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_WANT_GROW, 1, null, null, null, 0, false));
        final JPanel panel2 = new JPanel();
        panel2.setLayout(new GridLayoutManager(1, 1, new Insets(0, 0, 0, 0), -1, -1));
        panel1.add(panel2, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        buttonOK = new JButton();
        this.$$$loadButtonText$$$(buttonOK, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.save"));
        buttonOK.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.save.tooltip"));
        buttonOK.putClientProperty("hideActionText", Boolean.FALSE);
        buttonOK.putClientProperty("html.disable", Boolean.TRUE);
        panel2.add(buttonOK, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final JPanel panel3 = new JPanel();
        panel3.setLayout(new GridLayoutManager(6, 2, new Insets(0, 0, 0, 0), -1, -1));
        contentPane.add(panel3, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
        panel3.setBorder(BorderFactory.createTitledBorder(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.settings")));
        final Spacer spacer2 = new Spacer();
        panel3.add(spacer2, new GridConstraints(5, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
        dontMerge32BitAndCheckBox = new JCheckBox();
        this.$$$loadButtonText$$$(dontMerge32BitAndCheckBox, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.mergeUpdates"));
        dontMerge32BitAndCheckBox.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.mergeUpdates.tooltip"));
        dontMerge32BitAndCheckBox.putClientProperty("html.disable", Boolean.TRUE);
        panel3.add(dontMerge32BitAndCheckBox, new GridConstraints(3, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        registerInstalledUpdateFilesCheckBox = new JCheckBox();
        this.$$$loadButtonText$$$(registerInstalledUpdateFilesCheckBox, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.registerToUninst"));
        registerInstalledUpdateFilesCheckBox.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.registerToUninst.tooltip"));
        registerInstalledUpdateFilesCheckBox.putClientProperty("html.disable", Boolean.TRUE);
        panel3.add(registerInstalledUpdateFilesCheckBox, new GridConstraints(4, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        skipChecksumValidationCheckBox = new JCheckBox();
        skipChecksumValidationCheckBox.setEnabled(true);
        skipChecksumValidationCheckBox.setSelected(false);
        this.$$$loadButtonText$$$(skipChecksumValidationCheckBox, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.skipValidation"));
        skipChecksumValidationCheckBox.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.skipValidation.tooltip"));
        skipChecksumValidationCheckBox.putClientProperty("html.disable", Boolean.TRUE);
        panel3.add(skipChecksumValidationCheckBox, new GridConstraints(2, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        extendedValidationCheckbox = new JCheckBox();
        extendedValidationCheckbox.setEnabled(true);
        extendedValidationCheckbox.setSelected(false);
        this.$$$loadButtonText$$$(extendedValidationCheckbox, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.extendedValidation"));
        extendedValidationCheckbox.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.extendedValidation.tooltip"));
        extendedValidationCheckbox.putClientProperty("html.disable", Boolean.TRUE);
        panel3.add(extendedValidationCheckbox, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        disableDiskSpaceCheckingCheckbox = new JCheckBox();
        disableDiskSpaceCheckingCheckbox.setEnabled(true);
        disableDiskSpaceCheckingCheckbox.setSelected(false);
        this.$$$loadButtonText$$$(disableDiskSpaceCheckingCheckbox, ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.disableDiskSpaceChecking"));
        disableDiskSpaceCheckingCheckbox.setToolTipText(ResourceBundle.getBundle("aura/cm3d2/ui/text").getString("SettingsDialog.disableDiskSpaceChecking.tooltip"));
        disableDiskSpaceCheckingCheckbox.putClientProperty("html.disable", Boolean.TRUE);
        panel3.add(disableDiskSpaceCheckingCheckbox, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
        final Spacer spacer3 = new Spacer();
        panel3.add(spacer3, new GridConstraints(0, 1, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_HORIZONTAL, GridConstraints.SIZEPOLICY_WANT_GROW, 1, null, null, null, 0, false));
    }

    /**
     * @noinspection ALL
     */
    private void $$$loadButtonText$$$(AbstractButton component, String text) {
        StringBuffer result = new StringBuffer();
        boolean haveMnemonic = false;
        char mnemonic = '\0';
        int mnemonicIndex = -1;
        for (int i = 0; i < text.length(); i++) {
            if (text.charAt(i) == '&') {
                i++;
                if (i == text.length()) break;
                if (!haveMnemonic && text.charAt(i) != '&') {
                    haveMnemonic = true;
                    mnemonic = text.charAt(i);
                    mnemonicIndex = result.length();
                }
            }
            result.append(text.charAt(i));
        }
        component.setText(result.toString());
        if (haveMnemonic) {
            component.setMnemonic(mnemonic);
            component.setDisplayedMnemonicIndex(mnemonicIndex);
        }
    }

    /**
     * @noinspection ALL
     */
    public JComponent $$$getRootComponent$$$() {
        return contentPane;
    }
}
